# -*- coding: utf-8 -*-
"""
Created on Sun Apr 19 09:14:34 2020

@author: stefa
"""


#TODO Konstanten und anpassbare Sachen hier auslagern einheitendatenbank


language = {"DE": 
                {"table": 
                     {"header": "Eingangsgrößen", "var": "Variable", "val": "Wert", "unit": "Einheit"}
                },
            "EN": 
                {"table": 
                     {"header": "input values", "var": "variable", "val": "value", "unit": "unit"}
                }
            }

accuracy = 3

mult_sign = "\\cdot" #others: \times, \star, \ast